import sqlite3

sqlite_file = 'alertInvest.sqlite'    # name of the sqlite database file

# Table with Bonds
#table_bonds = 'bonds'  # name of the table to be created
#new_field = 'my_1st_column' # name of the column

# Table with log of monitoring tasks
table_job_log = 'job_logs'  # name of the table to be created
columns = 'id INTEGER PRIMARY KEY AUTOINCREMENT, date_task TEXT, emailSent BOOLEAN' # name of the column

# Connecting to the database file
conn = sqlite3.connect(sqlite_file)
c = conn.cursor()

# Creating a new SQLite table with 1 column
c.execute('CREATE TABLE {tn} ({nf})'\
        .format(tn=table_job_log, nf=columns))

# Committing changes and closing the connection to the database file
conn.commit()
conn.close()


#!/home/fernandosjp/anaconda/bin/python

# Bond Scrapper
from lxml.html import parse
from lxml import etree
from urllib2 import urlopen
from pandas.io.parsers import TextParser
# Data
import pandas as pd
# Logging
import logging
#Regexp 
import re

class BondScrapper(object):
	"""
	BondScrapper is responsible for accessing Tesouro Direto website and returning a DataFrame containing the following columns:

	1. Titulo
	2. Vencimento
	3. Taxa Compra
	4. Taxa Venda
	5. Preco Compra
	6. Preco Venda

	"""

	def __unpack(self, row, kind='td'):
		elts = row.findall('.//%s' % kind)
		return [val.text_content() for val in elts]

	def __parse_options_data(self, table):
		rows = table.findall('.//tr')
		#TODO: understand how to make header parser generic
		#header = __unpack(rows[0], kind='th')
		header = ['titulo','vencimento','taxa_compra','taxa_venda','preco_compra','preco_venda']
		data = [self.__unpack(r) for r in rows[2:]]
		return TextParser(data, names=header).get_chunk()

	def __convertToInt(self, x):
		"""
		Function to convert yield to integer
		"""
		try:
			x=float(x.replace(',','.'))
		except Exception, e:
			x=0
		return x

	def __extractBondName(self, x):
	    searchString = re.search(r"\(([^)]+)\)", str(x))
	    bondName=""
	    try:
	        bondName = searchString.group(1)
	    except:
	        pass
	    
	    return bondName

	def getBondTable(self):
		"""
		Opens url and returns bond table in a Data Frame with the columns [titulo,vencimento,taxas]
		-----

		Return Data Frame
		
		"""
		#Open a browser
		url='http://www.tesouro.fazenda.gov.br/tesouro-direto-precos-e-taxas-dos-titulos'
		#logger.info('Opening url: {}'.format(url))
		parsed = parse(urlopen('http://www.tesouro.fazenda.gov.br/tesouro-direto-precos-e-taxas-dos-titulos'))
		doc = parsed.getroot()
		tables = doc.findall('.//table')

		df = self.__parse_options_data(tables[1])
		df.dropna(inplace=True)

		#logger.info('Correcting Bond Yields')
		df['taxa'] = df.apply(lambda row: self.__convertToInt(row['taxa_compra']), axis=1)
		df['titulo'] = df.apply(lambda row: self.__extractBondName(row['titulo']), axis=1)
		
		columns = ['titulo','vencimento','taxa']

		return df[columns]







import click
from alerter import Alerter

@click.group()
def cli():
    pass

@cli.command()
@click.argument('config', default="config.yml",type=click.Path(exists=True))
@click.argument('alerts', default="alerts.json", type=click.Path(exists=True))
def alert(**kwargs):
	alert = Alerter(kwargs['alerts'],kwargs['config'])
	alert.alert()

if __name__ == '__main__':
    cli()





#!/home/fernandosjp/anaconda/bin/python

# Data
import pandas as pd
import sqlite3
# Emails
from emailSender.sendMail import Email
# Logging
import logging
#Others
import datetime
import json
import yaml
from  scrapper import BondScrapper

#Logging
# create logger with 'alert tesouro'
logger = logging.getLogger('alert_tesouro')
logger.setLevel(logging.DEBUG)
# create file handler which logs even debug messages
fh = logging.FileHandler('alertInvest.log')
fh.setLevel(logging.DEBUG)
# create file handler which logs even debug messages
eh = logging.FileHandler('alertInvestError.log')
eh.setLevel(logging.ERROR)
# create console handler with a higher log level
ch = logging.StreamHandler()
ch.setLevel(logging.INFO)
# create formatter and add it to the handlers
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
fh.setFormatter(formatter)
ch.setFormatter(formatter)
eh.setFormatter(formatter)
# add the handlers to the logger
logger.addHandler(fh)
logger.addHandler(ch)
logger.addHandler(eh)

class Alerter(object):
	"""
	Alert class is responsible for looping into predefined alerts and sending e-mails.
	"""
	def __init__(self, alerts, config):
		"""
		import alerts and config
		"""
		with open(alerts) as json_data_file:
		    self.alerts = json.load(json_data_file)

		# Load configuration
		with open(config, 'r') as f:
			self.config = yaml.safe_load(f)

	def saveTask(self, emailSent):
		# Connecting to the database file
		sqlite_file = 'alertInvest.sqlite'  
		conn = sqlite3.connect(sqlite_file)
		c = conn.cursor()

		now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
		
		try:
			c.execute("INSERT INTO job_logs (date_task, emailSent) VALUES (?,?)", (now,emailSent))
			logger.info('Successfully inserted in database!')
		except Exception, e:
			logger.error('Not able to INSERT in database!')

		conn.commit()
		conn.close()

		return None

	def sendEmailAlert(self):

		email = Email(self.config['smtp'])
		msgText = ""
		subject = "[Tesouro Direto] Alerta COMPRA TUDO - {}".format(self.alertQuery)
		msgText=self.bonds_table.to_html()

		email.sendEmail(subject, msgText, sender = 'Tesouro Invest')

		return None

	def alert_trigger(self, alert):
		
		logger.info('Check condition: bond {BOND} and yield {YIELD}'.format(BOND = alert['bound_name'], YIELD = alert['yield']))
		
		self.alertQuery = "(taxa>={YIELD}) and (titulo=='{BOND}') ".format(YIELD = alert['yield'],
																		   BOND = alert['bound_name'])
		emailSent = False

		if not self.bonds_table.query(self.alertQuery).empty:
			logger.info('Condition satisfied!!')
			try:
				logger.info('Sending Email...')
				self.sendEmailAlert()	
			except Exception, e:
				logger.info('Email not sent...')
				raise e
			finally:
				emailSent = True
				logger.info('Email sent!')
		else:
			logger.info('No bonds today!!')

		logger.info('Saving task in DB...')
		self.saveTask(emailSent)

		return None		

	def alert(self):
        
		scrapper = BondScrapper()
		self.bonds_table = scrapper.getBondTable()

		for alert in self.alerts['alerts']:
			self.alert_trigger(alert)
	





import sys,os,re
import mimetypes
from time import sleep
from smtplib import SMTP
from email.MIMEMultipart import MIMEMultipart
from email.MIMEText import MIMEText
from email.MIMEImage import MIMEImage
from email.MIMEBase import MIMEBase
from email import Encoders
from email import encoders
from email.utils import COMMASPACE, formatdate

class Email(object):
    def __init__(self, config):
        self.config = config
        
    def sendEmail(self, subject, msgText, sender = 'Standard Sender', imgNames=None, imagesInMsgBody = False, replyTo=None, maxWidth="600px"):
        """
        Sends an e-mail with or without images attached.

        Parameters
        ----------
        subject : e-mail subject
        msgText : body of the e-mail in html
        imgNames : list of images to be attached, default None
        imagesInMsgBody: if True leave a key {IMAGES} in the place you want the images to be placed, default False
        maxWidth : max width of the images attached

        Returns
        -------
        

        """
    
        try:
            conn = SMTP(self.config['server'], self.config['port'])

            msg = MIMEMultipart()
            msg['Subject']= subject
            msg['From']   = sender
            msg['cc'] = ''

            if replyTo:
                msg['reply-to'] = replyTo

            #Attach images
            imagesHtml = ""
            if imgNames:
                i = 0
                for imgName in imgNames:
                    fp = open(imgName, 'rb')
                    img = MIMEImage(fp.read())
                    img.add_header('Content-Id', '<image_{i}>'.format(i = i))
                    imagesHtml += '<p><img style="max-width:{MAX_WIDTH}; width:100%" src="cid:image_{i}" /></p>'.format(MAX_WIDTH=maxWidth,i = i)
                    fp.close()
                    msg.attach(img)
                    i += 1
            
            body=""

            #Place images in HTML body in presence of {IMAGES}
            if imagesInMsgBody:
                try:
                    body = msgText.format(IMAGES=imagesHtml)
                except Exception, e:
                    pass
            else:
                body+=imagesHtml
                body+=msgText

            msg.attach(MIMEText(body, _subtype='html'))

            conn.ehlo()
            conn.starttls()
            conn.set_debuglevel(False)
            conn.login(self.config['username'],self.config['password'])
            try:
                conn.sendmail(sender, self.config['to'], msg.as_string())
            finally:
                conn.close()
        except:
            print "Unexpected error:", sys.exc_info()[0]